<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXPoll
 * @subpackage CBXPoll/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    CBXPoll
 * @subpackage CBXPoll/admin
 * @author     codeboxr <info@codeboxr.com>
 */
class CBXPoll_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;


	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    Setting api instance.
	 */
	private $settings_api;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->settings_api = new CBXPoll_Settings();
	}//end constructor

	public function init_cbxpoll_type(){
		CBXPollHelper::create_cbxpoll_post_type();
	}//end method init_cbxpoll_type

	/**
	 * Register and enqueue admin-specific style sheet.
	 *
	 *
	 * @since     1.0.0
	 *
	 * @return    null    Return early if no settings page is registered.
	 */
	public function enqueue_styles($hook){
		global $post_type;


		//register css files
		wp_register_style('cbxpoll-chosen', plugins_url('../assets/css/chosen.min.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);
		wp_register_style('cbxpoll-ui-styles', plugins_url('../assets/css/ui-lightness/jquery-ui.min.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);
		wp_register_style('cbxpoll-ui-styles-timepicker', plugins_url('../assets/js/jquery-ui-timepicker-addon.min.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);

		wp_register_style('cbxpoll-ply-styles', plugins_url('../assets/css/ply.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);
		wp_register_style('cbxpoll-switchery-styles', plugins_url('../assets/css/switchery.min.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);
		wp_register_style('cbxpoll-admin-styles', plugins_url('../assets/css/cbxpoll_admin.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);


		if ((in_array($hook, array('edit.php', 'post.php', 'post-new.php')) && 'cbxpoll' == $post_type) || ($hook == 'cbxpoll_page_cbxpollsetting')) {
			//now enqueue css files
			wp_enqueue_style('cbxpoll-chosen');
			wp_enqueue_style('cbxpoll-ui-styles');
			wp_enqueue_style('cbxpoll-ui-styles-timepicker');

			wp_enqueue_style('cbxpoll-ply-styles');
			wp_enqueue_style('cbxpoll-switchery-styles');
			wp_enqueue_style('cbxpoll-admin-styles');


			do_action('cbxpolladmin_custom_style');
		}

		if($hook == 'cbxpoll_page_cbxpollsetting'){

			wp_enqueue_style('wp-color-picker');


			wp_enqueue_script('jquery');
			wp_enqueue_script('wp-color-picker');
			wp_enqueue_media();



			wp_register_style('cbxpoll-admin-setting', plugins_url('../assets/css/cbxpoll-admin-setting.css', __FILE__), array(), CBX_POLL_PLUGIN_VERSION);
			wp_enqueue_style('cbxpoll-admin-setting');

			wp_register_script('cbxpoll-admin-setting', plugins_url('../assets/js/cbxpoll-admin-setting.js', __FILE__), array('jquery'), CBX_POLL_PLUGIN_VERSION);
			wp_enqueue_script('cbxpoll-admin-setting');
		}
	}


	/**
	 * Register and enqueue admin-specific JavaScript.
	 *
	 * @since     1.0.0
	 *
	 * @return    null    Return early if no settings page is registered.
	 */
	public function enqueue_scripts($hook)
	{
		global $post_type;

		wp_register_script('cbxpoll-jseventManager', plugins_url('../assets/js/cbxpolljsactionandfilter.js', __FILE__), array(), CBX_POLL_PLUGIN_VERSION, true);
		wp_register_script('cbxpoll-choosen-script', plugins_url('../assets/js/chosen.jquery.min.js', __FILE__), array('jquery'), CBX_POLL_PLUGIN_VERSION, true);

		wp_register_script('cbxpoll-ui-time-script', plugins_url('../assets/js/jquery-ui-timepicker-addon.js', __FILE__), array('jquery', 'jquery-ui-datepicker'), CBX_POLL_PLUGIN_VERSION, true);
		wp_register_script('cbxpoll-clipboardjs', plugins_url('../assets/js/clipboard.min.js', __FILE__), array('jquery'), CBX_POLL_PLUGIN_VERSION, true);
		wp_register_script('cbxpoll-plyjs', plugins_url('../assets/js/ply.min.js', __FILE__), array('jquery'), CBX_POLL_PLUGIN_VERSION, true);
		wp_register_script('cbxpoll-switcheryjs', plugins_url('../assets/js/switchery.min.js', __FILE__), array('jquery'), CBX_POLL_PLUGIN_VERSION, true);


		wp_register_script('cbxpoll-admin-script', plugins_url('../assets/js/cbxpoll_admin.js', __FILE__), array('jquery', 'cbxpoll-clipboardjs','cbxpoll-switcheryjs', 'cbxpoll-plyjs', 'cbxpoll-ui-time-script', 'cbxpoll-choosen-script', 'cbxpoll-jseventManager'), CBX_POLL_PLUGIN_VERSION, true);

		wp_register_script('cbxpolladminsingle', plugins_url('../assets/js/cbxpoll-admin-single.js', __FILE__), array('jquery', 'cbxpoll-clipboardjs', 'cbxpoll-switcheryjs', 'cbxpoll-plyjs', 'cbxpoll-ui-time-script', 'cbxpoll-choosen-script' , 'cbxpoll-jseventManager'), CBX_POLL_PLUGIN_VERSION, true);




		//if ((in_array($hook, array('edit.php', 'post.php', 'post-new.php')) && 'cbxpoll' == $post_type) || ($hook == 'cbxpollsetting')) {

		//backend poll listing
		if ((in_array($hook, array('edit.php')) && 'cbxpoll' == $post_type) || ($hook == 'cbxpoll_page_cbxpollsetting')) {

			wp_enqueue_script('cbxpoll-jseventManager');
			wp_enqueue_script('jquery');
			wp_enqueue_style('wp-color-picker');
			wp_enqueue_style('thickbox');
			wp_enqueue_script('wp-color-picker');
			wp_enqueue_script('media-upload');

			wp_enqueue_script('cbxpoll-choosen-script');
			wp_enqueue_script('cbxpoll-ui-time-script');
			wp_enqueue_script('cbxpoll-clipboardjs');
			wp_enqueue_script('cbxpoll-plyjs');
			wp_enqueue_script('cbxpoll-switcheryjs');
			wp_enqueue_script('cbxpoll-admin-script');

			do_action('cbxpolladmin_custom_script');
		}


		//backend single post
		if (in_array($hook, array('post.php', 'post-new.php')) && 'cbxpoll' == $post_type) {

			if ( ! class_exists( '_WP_Editors', false ) ){
				require( ABSPATH . WPINC . '/class-wp-editor.php' );
			}


			//adding translation and other variables from php to js for single post edit screen
			$cbxpolladminsingle_arr = array(
				'remove_label'      => esc_html__('Remove', 'cbxpoll'),
				'move_label'        => esc_html__('Move', 'cbxpoll'),
				'move_title'        => esc_html__('Drag and Drop to reorder answers', 'cbxpoll'),
				'answer_label'      => esc_html__('Answer', 'cbxpoll'),
				'deleteconfirm'       => esc_html__('Are you sure to delete this item?', 'cbxpoll'),
				'deleteconfirmok'     => esc_html__('Sure', 'cbxpoll'),
				'deleteconfirmcancel' => esc_html__('Oh! No', 'cbxpoll'),
				'ajaxurl'             => admin_url('admin-ajax.php'),
				'nonce'               => wp_create_nonce('cbxpoll'),
				'teeny_editor_settings' => array(
					'teeny' => true,
					'textarea_name' => '',
					'textarea_rows' => 10,
					'media_buttons' => FALSE,
					'editor_class'  => ''
				)
			);

			wp_localize_script('cbxpolladminsingle', 'cbxpolladminsingleObj', $cbxpolladminsingle_arr );

			wp_enqueue_script('cbxpoll-jseventManager');
			wp_enqueue_script('jquery');
			wp_enqueue_style('wp-color-picker');
			wp_enqueue_style('thickbox');
			wp_enqueue_script('wp-color-picker');
			wp_enqueue_script('media-upload');

			wp_enqueue_style('jquery-ui-core'); //jquery ui core
			wp_enqueue_style('jquery-ui-datepicker'); //jquery ui datepicker
			wp_enqueue_style('jquery-ui-sortable'); //jquery ui sortable

			wp_enqueue_script('cbxpoll-choosen-script');
			wp_enqueue_script('cbxpoll-ui-time-script');
			wp_enqueue_script('cbxpoll-clipboardjs');
			wp_enqueue_script('cbxpoll-plyjs');
			wp_enqueue_script('cbxpoll-switcheryjs');

			wp_enqueue_script('cbxpolladminsingle');

			do_action('cbxpolladmin_single_custom_script');
		}

	}//end method enqueue_scripts

	/**
	 * on admin init initialize setting and handle cbxpoll type post delete
	 */
	public function admin_init(){

		//init setting api
		$this->settings_api->set_sections($this->get_setting_sections());
		$this->settings_api->set_fields($this->get_setting_fields());

		//initialize them
		$this->settings_api->admin_init();


		//handle cbxpoll type post delete
		add_action('delete_post', array($this, 'cbxpoll_delete_vote'), 10);
	}

	/**
	 * Delete vote on poll type post delete
	 *
	 * @param type $postid
	 */
	function cbxpoll_delete_vote($poll_id)
	{
		global $wpdb;

		$post_type = get_post_type($poll_id);
		if ($post_type !== false && $post_type == 'cbxpoll') {
			$poll_table = CBXPollHelper::cbx_poll_table_name();

			$poll_votes = CBXPollHelper::getAllVotes('id', 'desc', -1, 1, $poll_id);
			if(is_array($poll_votes) && sizeof($poll_votes) > 0){
				//let's delete all single vote and allow hooks
				foreach ($poll_votes as $log_info){
					$id = intval($log_info['id']);
					$sql    = $wpdb->prepare("DELETE FROM $poll_table WHERE id=%d", $id);
					$delete_status = $wpdb->query($sql);

					do_action( 'cbxpoll_vote_delete_before', $log_info );

					if($delete_status !== false){
						do_action( 'cbxpoll_vote_delete_after', $log_info );

						if (intval($log_info['published']) == 1) {
							//if already published vote then readjust the total votes
							$poll_total = intval(get_post_meta($poll_id, '_cbxpoll_total_votes', true)); //at least a single vote
							$poll_voted = intval(get_post_meta($poll_id, '_cbxpoll_is_voted', true)); //at least a single vote

							$poll_total = $poll_total - 1;
							update_post_meta($poll_id, '_cbxpoll_total_votes', $poll_total); //can help for showing most voted oll //meta added
							if ($poll_total == 0) {
								update_post_meta($poll_id, '_cbxpoll_is_voted', 0);
							}
						}
					}//end if delete success
				}
			}//end if array
		}
	}

	/**
	 * On user delete delete user's vote
	 *
	 * @param $user_id
	 */
	public function on_user_delete_vote_delete($user_id){
		global $wpdb;

		$user_id = intval($user_id);
		$poll_table = CBXPollHelper::cbx_poll_table_name();

		$poll_votes = CBXPollHelper::getAllVotesByUser($user_id,'id', 'desc', -1, 1);

		if(is_array($poll_votes) && sizeof($poll_votes) > 0){
			//let's delete all single vote and allow hooks
			foreach ($poll_votes as $log_info){
				$id = intval($log_info['id']);
				$poll_id = intval($log_info['poll_id']);
				$sql    = $wpdb->prepare("DELETE FROM $poll_table WHERE id=%d", $id);
				$delete_status = $wpdb->query($sql);

				do_action( 'cbxpoll_vote_delete_before', $log_info );

				if($delete_status !== false){
					do_action( 'cbxpoll_vote_delete_after', $log_info );

					if (intval($log_info['published']) == 1) {
						//if already published vote then readjust the total votes
						$poll_total = intval(get_post_meta($poll_id, '_cbxpoll_total_votes', true)); //at least a single vote
						$poll_voted = intval(get_post_meta($poll_id, '_cbxpoll_is_voted', true)); //at least a single vote

						$poll_total = $poll_total - 1;
						update_post_meta($poll_id, '_cbxpoll_total_votes', $poll_total); //can help for showing most voted oll //meta added
						if ($poll_total == 0) {
							update_post_meta($poll_id, '_cbxpoll_is_voted', 0);
						}
					}
				}//end if delete success
			}
		}//end if array
	}//end method on_user_delete_vote_delete

	/**
	 * CBX Poll Core Global Setting Sections
	 *
	 * @return mixed|void
	 */
	public function get_setting_sections()
	{
		$sections = array(
			array(
				'id'    => 'cbxpoll_global_settings',
				'title' => esc_html__('Poll Default Settings', 'cbxpoll')
			),
			array(
				'id'    => 'cbxpoll_email_setting',
				'title' => esc_html__('Email Setting', 'cbxpoll')
			),
			array(
				'id'    => 'cbxpoll_tools',
				'title' => esc_html__('Tools', 'cbxpoll')
			)
		);

		return apply_filters('cbxpoll_setting_sections', $sections);
	}//end method get_setting_sections

	/**
	 * CBX Poll Setting Core Fields
	 *
	 * @return mixed|void
	 */
	public function get_setting_fields(){

		$roles = CBXPollHelper::user_roles(false, true); //

		$reset_data_link = add_query_arg('cbxpoll_fullreset', 1, admin_url('edit.php?post_type=cbxpoll&page=cbxpollsetting'));

		$table_names = CBXPollHelper::getAllDBTablesList();
		$table_html = '<p id="cbxpoll_plg_gfig_info"><strong>'.esc_html__('Following database tables will be reset/deleted.', 'cbxwpsimpleaccounting').'</strong></p>';

		$table_counter = 1;
		foreach($table_names as $key => $value){
			$table_html .= '<p>'.str_pad($table_counter, 2, '0', STR_PAD_LEFT).'. '.$key.' - (<code>'.$value.'</code>)</p>';
			$table_counter++;
		}

		$table_html .= '<p><strong>'.esc_html__('Following option values created by this plugin(including addon) from wordpress core option table','cbxwpsimpleaccounting').'</strong></p>';


		$option_values = CBXPollHelper::getAllOptionNames();
		$table_counter = 1;
		foreach($option_values as $key => $value){
			$table_html .= '<p>'.str_pad($table_counter, 2, '0', STR_PAD_LEFT).'. '.$value['option_name'].' - '.$value['option_id'].' - (<code style="overflow-wrap: break-word; word-break: break-all;">'.$value['option_value'].'</code>)</p>';

			$table_counter++;
		}




		$poll_display_methods = CBXPollHelper::cbxpoll_display_options();
		$poll_display_methods = CBXPollHelper::cbxpoll_display_options_linear($poll_display_methods);


		$fields = array(
			'cbxpoll_global_settings' => apply_filters('cbxpoll_global_general_fields', array(
				'result_chart_type' => array(
					'name'    => 'result_chart_type',
					'label'   => esc_html__('Result Chart Style', 'cbxpoll'),
					'desc'    => __('Poll result display styles, text and polar area display type are free, you can buy more display option from <a href="https://codeboxr.com/product/cbx-poll-for-wordpress/" target="_blank">here</a>', 'cbxpoll'),
					'type'    => 'select',
					'default' => 'text',
					'options' => $poll_display_methods,
				),
				'user_roles'        => array(
					'name'     => 'user_roles',
					'label'    => esc_html__('Who Can Vote', 'cbxpoll'),
					'desc'     => esc_html__('which user role will have vote capability', 'cbxpoll'),
					'type'     => 'multiselect',
					//'optgroup' => 0,
					'default'  => array('administrator', 'editor', 'author', 'contributor', 'subscriber', 'guest'),
					'options'     => $roles,
					'optgroup'	=> 1,
					'placeholder' => esc_html__('Select user roles', 'cbxpoll')
				),

				'content'                   => array(
					'name'    => 'content',
					'label'   => esc_html__('Show Poll Description', 'cbxpoll'),
					'desc'    => esc_html__('Show description from poll post type', 'cbxpoll'),
					'type'    => 'radio',
					'default' => 1,
					'options' => array(
						'1' => esc_html__('Yes', 'cbxpoll'),
						'0' => esc_html__('No', 'cbxpoll')
					)
				),
				'never_expire'              => array(
					'name'    => 'never_expire',
					'label'   => esc_html__('Never Expire', 'cbxpoll'),
					'desc'    => esc_html__('If set polls will never expire. You can also set individual poll end time.', 'cbxpoll'),
					'type'    => 'radio',
					'default' => 0,
					'options' => array(
						'1' => esc_html__('Yes', 'cbxpoll'),
						'0' => esc_html__('No', 'cbxpoll')
					)
				),
				'show_result_before_expire' => array(
					'name'    => 'show_result_before_expire',
					'label'   => esc_html__('Show result before expires', 'cbxpoll'),
					'desc'    => esc_html__('Select if you want poll to show result before expires. After expires the result will be shown always. Please check it if poll never expires.', 'cbxpoll'),
					'type'    => 'radio',
					'default' => 1, //new change 0 -> 1
					'options' => array(
						'1' => esc_html__('Yes', 'cbxpoll'),
						'0' => esc_html__('No', 'cbxpoll')
					)
				),
				/*'show_result_all'           => array(
					'name'    => 'show_result_all',
					'label'   => esc_html__('Show result to all', 'cbxpoll'),
					'desc'    => esc_html__('Check this if you want to show result to them who can not vote.', 'cbxpoll'),
					'type'    => 'radio',
					'default' => 0,
					'options' => array(
						'1' => esc_html__('Yes', 'cbxpoll'),
						'0' => esc_html__('No', 'cbxpoll')
					)
				),*/
				'cookiedays'                => array(
					'name'        => 'cookiedays',
					'label'       => esc_html__('Cookie Expiration Days', 'cbxpoll'),
					'desc'        => esc_html__('For guest user cookie is placed in browser, For how many days cookie will not expire. Default is 30 days', 'cbxpoll'),
					'type'        => 'number',
					'default'     => '30',
					'placeholder' => esc_html__('Number of days', 'cbxpoll')

				),
				'logmethod'                 => array(
					'name'    => 'logmethod',
					'label'   => esc_html__('Log Method', 'cbxpoll'),
					'desc'    => __('Logging method. [<strong> Note:</strong> Please Select at least one or a guest user will vote multiple time for a poll.]', 'cbxpoll'),
					'type'    => 'select',
					'default' => 'both',
					'options' => array(
						'ip'     => esc_html__('IP', 'cbxpoll'),
						'cookie' => esc_html__('Cookie', 'cbxpoll'),
						'both'   => esc_html__('Both(IP or cookie any one)', 'cbxpoll'),
					)
				),
				'answer_grid_list'           => array(
					'name'    => 'answer_grid_list',
					'label'   => esc_html__('Answer Display Format', 'cbxpoll'),
					'desc'    => esc_html__('Traditionally answer is shown as vericala list but sometimes grid presentation better for user experience.', 'cbxpoll'),
					'type'    => 'radio',
					'default' => 0,
					'options' => array(
						'0' => esc_html__('List', 'cbxpoll'),
						'1' => esc_html__('Grid', 'cbxpoll')
					)
				),
			)),
			'cbxpoll_email_setting'   => apply_filters('cbxpoll_global_email_fields', array(
				'headerimage' => array(
					'name'     => 'headerimage',
					'label'    => esc_html__('Header Image', 'cbxpoll'),
					'desc'     => esc_html__('Url To email you want to show as email header.Upload Image by media uploader.', 'cbxpoll'),
					'type'     => 'file',
					'default'  => '',
					'desc_tip' => true,
				),

				'basecolor'           => array(
					'name'     => 'basecolor',
					'label'    => esc_html__('Base Color', 'cbxpoll'),
					'desc'     => esc_html__('The base color of the email.', 'cbxpoll'),
					'type'     => 'color',
					'default'  => '#557da1',
					'desc_tip' => true,
				),
				'backgroundcolor'     => array(
					'name'     => 'backgroundcolor',
					'label'    => esc_html__('Background Colour', 'cbxpoll'),
					'desc'     => esc_html__('The background color of the email.', 'cbxpoll'),
					'type'     => 'color',
					'default'  => '#f5f5f5',
					'desc_tip' => true,
				),
				'bodybackgroundcolor' => array(
					'name'     => 'bodybackgroundcolor',
					'label'    => esc_html__('Body Background Color', 'cbxpoll'),
					'desc'     => esc_html__('The background colour of the main body of email.', 'cbxpoll'),
					'type'     => 'color',
					'default'  => '#fdfdfd',
					'desc_tip' => true,
				),
				'bodytextcolor'       => array(
					'name'     => 'bodytextcolor',
					'label'    => esc_html__('Body Text Color', 'cbxpoll'),
					'desc'     => esc_html__('The body text colour of the main body of email.', 'cbxpoll'),
					'type'     => 'color',
					'default'  => '#505050',
					'desc_tip' => true,
				),
				'footertext'          => array(
					'name'     => 'footertext',
					'label'    => esc_html__('Footer Text', 'cbxpoll'),
					'desc'     => esc_html__('The text to appear at the email footer. Syntax available - {sitename}', 'cbxpoll'),
					'type'     => 'wysiwyg',
					'default'  => '{sitename}',
					'desc_tip' => true,
				)
			)),
			'cbxpoll_tools'    => apply_filters('cbxpoll_global_tools_fields', array(
					'delete_global_config' =>  array(
						'name'     => 'delete_global_config',
						'label'    => esc_html__('On Uninstall delete plugin data', 'cbxpoll'),
						'desc'     => '<p>'.__('Delete Global Config data and custom table created by this plugin on uninstall.', 'cbxpoll').' '. __('Details table information is <a href="#cbxpoll_plg_gfig_info">here</a>', 'cbxpoll').'</p>'.'<p>'.__('<strong>Please note that this process can not be undone and it is recommended to keep full database backup before doing this.</strong>', 'cbxpoll').'</p>',
						'type'     => 'radio',
						'options'  => array(
							'yes' => esc_html__('Yes', 'cbxpoll'),
							'no'  => esc_html__('No', 'cbxpoll'),
						),
						'default'  => 'no',
						'desc_tip' => true,
					),
					'reset_data'           => array(
						'name'     => 'reset_data',
						'label'    => esc_html__('Reset all data', 'cbxpoll'),
						'desc'     => sprintf(__('Reset option values and all tables created by this plugin. 
<a class="button button-primary" onclick="return confirm(\'%s\')" href="%s">Reset Data</a>', 'cbxpoll'), esc_html__('Are you sure to reset all data, this process can not be undone?', 'cbxpoll') ,$reset_data_link).$table_html,
						'type'     => 'html',
						'default'  => 'off',
						'desc_tip' => true,
					)
				)
			)
		);

		return apply_filters('cbxpoll_global_fields', $fields);
	}//end method get_setting_fields

	/**
	 *  add setting page menu
	 */
	public function admin_menu(){
		$setting_page_hook =  add_submenu_page('edit.php?post_type=cbxpoll', esc_html__('Poll Settings', 'cbxpoll'), esc_html__('Settings', 'cbxpoll'), 'manage_options',  'cbxpollsetting', array($this, 'admin_menu_setting_page'));
	}//end method admin_menu

	/**
	 * Render the settings page for this plugin.
	 *
	 * @since    1.0.0
	 */
	public function admin_menu_setting_page()
	{
		$plugin_data = get_plugin_data(plugin_dir_path(__DIR__) . '/../' . CBX_POLL_PLUGIN_BASE_NAME);
		include('partials/admin-settings-display.php');
	}//end method admin_menu_setting_page

	/**
	 * cbxpoll type post listing extra cols
	 *
	 * @param $cbxpoll_columns
	 * @return mixed
	 *
	 */
	public function add_new_poll_columns($cbxpoll_columns)
	{
		$cbxpoll_columns['title']      = esc_html__('Poll Title', 'cbxpoll');
		$cbxpoll_columns['pollstatus'] = esc_html__('Status', 'cbxpoll');
		$cbxpoll_columns['startdate']  = esc_html__('Start Date', 'cbxpoll');
		$cbxpoll_columns['enddate']    = esc_html__('End Date', 'cbxpoll');
		$cbxpoll_columns['date']       = esc_html__('Created', 'cbxpoll');
		$cbxpoll_columns['pollvotes']  = esc_html__('Votes', 'cbxpoll');
		$cbxpoll_columns['shortcode']  = esc_html__('Shortcode', 'cbxpoll');

		return $cbxpoll_columns;
	}//end method add_new_poll_columns

	/**
	 * cbxpoll type post listing extra col values
	 *
	 * @param $column_name
	 *
	 */
	public function manage_poll_columns($column_name)
	{

		global $post;

		$post_id = $post->ID;

		$end_date     = get_post_meta($post_id, '_cbxpoll_end_date', true);
		$start_date   = get_post_meta($post_id, '_cbxpoll_start_date', true);
		$never_expire = intval(get_post_meta($post_id, '_cbxpoll_never_expire', true));
		$total_votes  = intval(get_post_meta($post_id, '_cbxpoll_total_votes', true));

		switch ($column_name) {

			case 'pollstatus':
				// Get number of images in gallery
				if ($never_expire == 1) {
					if (new DateTime($start_date) > new DateTime()) {
						echo '<span class="dashicons dashicons-calendar"></span> ' . esc_html__('Yet to Start', 'cbxpoll'); //
					} else {
						echo '<span class="dashicons dashicons-yes"></span> ' . esc_html__('Active', 'cbxpoll');
					}

				} else {
					if (new DateTime($start_date) > new DateTime()) {
						echo '<span class="dashicons dashicons-calendar"></span> ' . __('Yet to Start', 'cbxpoll'); //
					} else if (new DateTime($start_date) <= new DateTime() && new DateTime($end_date) > new DateTime()) {
						echo '<span class="dashicons dashicons-yes"></span> ' . esc_html__('Active', 'cbxpoll');
					} else if (new DateTime($end_date) <= new DateTime()) {
						echo '<span class="dashicons dashicons-lock"></span> ' . esc_html__('Expired', 'cbxpoll');
					}
				}
				break;
			case 'startdate':
				echo $start_date;
				break;
			case 'enddate':
				echo $end_date;
				break;
			case 'pollvotes':
				echo apply_filters('cbxpoll_admin_listing_votes', $total_votes, $post_id);
				break;
			case 'shortcode':
				echo '<span data-clipboard-text=\'[cbxpoll id="' . $post_id . '"]\' title="' . esc_html__("Click to clipboard", "cbxpoll") . '" id="cbxpollshortcode-' . $post_id . '" class="cbxpollshortcode cbxpollshortcode-' . $post_id . '">[cbxpoll id="' . $post_id . '"]</span>';
				/*echo '<span class="cbxpollshortcodetrigger" data-clipboard-text=\'[cbxpoll id="' . $post_id . '"]\' title="' . __("Copy to clipboard", "cbxpoll") . '">
                        <img style="width: 16px;" src="' . plugins_url('assets/images/clippy.png', dirname(__FILE__)) . '" alt="' . __('Copy to clipboard', 'cbxpoll') . '"/>
                     </span>';*/
				break;
			default:
				break;
		} // end switch

	}//end method manage_poll_columns

	/**
	 * cbxpoll type post liting extra col sortable
	 *
	 * make poll table columns sortable
	 */
	function cbxpoll_columnsort($columns)
	{
		$columns['startdate']  = 'startdate';
		$columns['enddate']    = 'enddate';
		$columns['pollstatus'] = 'pollstatus';
		$columns['pollvotes']  = 'pollvotes';

		return $columns;
	}//end method cbxpoll_columnsort

	/**
	 * Hook custom meta box
	 */
	function metaboxes_display()
	{

		//add meta box in left side to show poll setting
		add_meta_box(
			'pollcustom_meta_box',                              // $id
			esc_html__('Poll Options', 'cbxpoll'),  // $title
			array($this, 'metabox_setting_display'),           // $callback
			'cbxpoll',                                      // $page
			'normal',                                           // $context
			'high');                                            // $priority

		//add meta box in right col to show the result
		add_meta_box(
			'pollresult_meta_box',                              // $id
			esc_html__('Poll Result', 'cbxpoll'),  // $title
			array($this, 'metabox_result_display'),           // $callback
			'cbxpoll',                                      // $page
			'side',                                           // $context
			'low');

		//add meta box in right col to show the result
		add_meta_box(
			'pollshortcode_meta_box',                              // $id
			esc_html__('Shortcode', 'cbxpoll'),  // $title
			array($this, 'metabox_shortcode_display'),           // $callback
			'cbxpoll',                                      // $page
			'side',                                           // $context
			'low');
	}//end method metaboxes_display

	/**
	 * Meta box display: Setting
	 */
	function metabox_setting_display(){

		global $post;
		$post_meta_fields = CBXPollHelper::get_meta_fields();


		$prefix         = '_cbxpoll_';
		$poll_postid    = 0;
		$answer_counter = 0;

		$is_voted     = 0;
		$poll_answers = array();
		$poll_colors  = array();


		if (isset($post->ID) && $post->ID > 0):
			$poll_postid = $post->ID;

			$is_voted               = intval(get_post_meta($poll_postid, $prefix . 'is_voted', true));
			$poll_answers           = get_post_meta($poll_postid, $prefix . 'answer', true);
			$poll_colors            = get_post_meta($poll_postid, $prefix . 'answer_color', true);
			$poll_answers_extra     = get_post_meta($poll_postid, $prefix . 'answer_extra', true);


			wp_nonce_field('cbxpoll_meta_box', 'cbxpoll_meta_box_nonce');

			echo '<div id="cbxpoll_answer_wrap" class="cbxpoll_answer_wrap" data-postid="'.$poll_postid.'">';
			echo '<h4>' . esc_html__('Poll Answers', 'cbxpoll') . '</h4>';
			echo __('<p>[<strong>Note : </strong>  <span>Please select different color for each field.]</span></p>', 'cbxpoll');


			echo '<ul id="cbx_poll_answers_items" class="cbx_poll_answers_items cbx_poll_answers_items_' . $post->ID . '">';


			if (is_array($poll_answers) && count($poll_answers) > 0) {

				foreach ((array)$poll_answers as $index => $poll_answer) {

					if (isset($poll_answer)) {
						$poll_answers_extra[$index] = isset($poll_answers_extra[$index])? $poll_answers_extra[$index]: array();

						echo CBXPollHelper::cbxpoll_answer_field_template($answer_counter, $poll_answer, $poll_colors[$index], $is_voted, $poll_answers_extra[$index], $poll_postid);

						$answer_counter++;
					}
				}
			} else {

				$answer_counter         = 3;
				$is_voted               = 0;
				$default_answers_titles = array(
					esc_html__('Yes', 'cbxpoll'), esc_html__('No', 'cbxpoll'), esc_html__('No comments', 'cbxpoll')
				);

				$default_answers_colors = array(
					'#2f7022', '#dd6363', '#e4e4e4'
				);

				$answers_extra = array('type' => 'default');

				for ($i = 0; $i < $answer_counter; $i++) {
					echo CBXPollHelper::cbxpoll_answer_field_template($i, $default_answers_titles[$i], $default_answers_colors[$i], $is_voted, $answers_extra, $poll_postid);
				}
			}
			echo '</ul>';
			?>
			<?php if (!$is_voted): ?>
			<div class="add-cbx-poll-answer-wrap" data-busy="0" data-postid="<?php echo $poll_postid; ?>" data-answercount="<?php echo intval($answer_counter); ?>">
				<a data-type="default"  id="add-cbx-poll-answer-default" class=" button button-primary add-cbx-poll-answer add-cbx-poll-answer-default add-cbx-poll-answer-<?php echo $poll_postid; ?>"><i style="line-height: 25px;" class="dashicons dashicons-media-text"></i> <?php echo esc_html__('Add Text Answer', 'cbxpoll'); ?></a>
				<?php  do_action('cbxpolladmin_add_answertype', $poll_postid, $answer_counter); ?>
			</div>
		<?php endif; ?>
			<br/>


			<?php
			echo '</div>';


			echo '<table class="form-table">';

			foreach ($post_meta_fields as $field) {

				$meta = get_post_meta($poll_postid, $field['id'], true);


				if ($meta == '' && isset($field['default'])) {

					$meta = $field['default'];
				}

				$label = isset($field['label']) ? $field['label'] : '';

				echo '<tr>';
				echo '<th><label for="' . $field['id'] . '">' . $label . '</label></th>';
				echo '<td>';


				switch ($field['type']) {

					case 'text':
						echo '<input type="text" class="regular-text" name="' . $field['id'] . '" id="' . $field['id'] . '-text-' . $poll_postid . '" value="' . $meta . '" size="30" />
			            <span class="description">' . $field['desc'] . '</span>';
						break;
					case 'number':
						echo '<input type="number" class="regular-text" name="' . $field['id'] . '" id="' . $field['id'] . '-number-' . $poll_postid . '" value="' . $meta . '" size="30" />
			            <span class="description">' . $field['desc'] . '</span>';
						break;

					case 'date':

						echo '<input type="text" class="cbxpollmetadatepicker" name="' . $field['id'] . '" id="' . $field['id'] . '-date-' . $poll_postid . '" value="' . $meta . '" size="30" />
			            <span class="description">' . $field['desc'] . '</span>';
						break;

					case 'colorpicker':


						echo '<input type="text" class="cbxpoll-colorpicker" name="' . $field['id'] . '" id="' . $field['id'] . '-date-' . $poll_postid . '" value="' . $meta . '" size="30" />
			             <span class="description">' . $field['desc'] . '</span>';
						break;

					case 'multiselect':
						echo '<select name="' . $field['id'] . '[]" id="' . $field['id'] . '-chosen-' . $poll_postid . '" class="chosen-select" multiple="multiple">';
						if (isset($field['optgroup']) && intval($field['optgroup'])) {

							foreach ($field['options'] as $optlabel => $data) {
								echo '<optgroup label="' . $optlabel . '">';
								foreach ($data as $key => $val)
									echo '<option value="' . $key . '"', is_array($meta) && in_array($key, $meta) ? ' selected="selected"' : '', ' >' . $val . '</option>';
								echo '<optgroup>';
							}

						} else {
							foreach ($field['options'] as $key => $val)
								echo '<option value="' . $key . '"', is_array($meta) && in_array($key, $meta) ? ' selected="selected"' : '', ' >' . $val . '</option>';
						}


						echo '</select><span class="description">' . $field['desc'] . '</span>';
						break;

					case 'select':
						echo '<select name="' . $field['id'] . '" id="' . $field['id'] . '-select-' . $poll_postid . '" class="cb-select select-' . $poll_postid . '">';

						if (isset($field['optgroup']) && intval($field['optgroup'])) {

							foreach ($field['options'] as $optlabel => $data) {
								echo '<optgroup label="' . $optlabel . '">';
								foreach ($data as $index => $option) {
									echo '<option ' . (($meta == $index) ? ' selected="selected"' : '') . ' value="' . $index . '">' . $option . '</option>';
								}

							}
						} else {
							foreach ($field['options'] as $index => $option) {
								echo '<option ' . (($meta == $index) ? ' selected="selected"' : '') . ' value="' . $index . '">' . $option . '</option>';
							}
						}


						echo '</select><br/><span class="description">' . $field['desc'] . '</span>';
						break;
					case 'radio':

						echo '<fieldset>
								<legend class="screen-reader-text"><span>input type="radio"</span></legend>';
						foreach ($field['options'] as $key => $value) {
							echo '<label title="g:i a" for="' . $field['id'] . '-radio-' . $poll_postid . '-' . $key . '">
										<input id="' . $field['id'] . '-radio-' . $poll_postid . '-' . $key . '" type="radio" name="' . $field['id'] . '" value="' . $key . '" ' . (($meta == $key) ? '  checked="checked" ' : '') . '  />
										<span>' . $value . '</span>
									</label><br>';


						}
						echo '</fieldset>';
						echo '<br/><span class="description">' . $field['desc'] . '</span>';
						break;

					case 'checkbox':
						echo '<input type="checkbox" name="' . $field['id'] . '" id="' . $field['id'] . '-checkbox-' . $poll_postid . '" class="cb-checkbox checkbox-' . $poll_postid . '" ', $meta ? ' checked="checked"' : '', '/>
                    <span for="' . $field['id'] . '">' . $field['desc'] . '</span>';
						break;
					case 'checkbox_group':
						if ($meta == '') {
							$meta = array();
							foreach ($field['options'] as $option) {
								array_push($meta, $option['value']);
							}
						}

						foreach ($field['options'] as $option) {
							echo '<input type="checkbox" value="' . $option['value'] . '" name="' . $field['id'] . '[]" id="' . $option['value'] . '-mult-chk-' . $poll_postid . '-field-' . $field['id'] . '" class="cb-multi-check mult-check-' . $poll_postid . '"', $meta && in_array($option['value'], $meta) ? ' checked="checked"' : '', ' />
                        <label for="' . $option['value'] . '">' . $option['label'] . '</label><br/>';
						}

						echo '<span class="description">' . $field['desc'] . '</span>';
						break;

				}
				echo '</td>';
				echo '</tr>';
			}
			echo '</table>';

		else:
			echo esc_html__('Please save the post once to enter poll answers.', 'cbxpoll');
		endif;

	}//end method metabox_setting_display

	/**
	 * Renders metabox in right col to show result
	 */
	function metabox_result_display()
	{

		global $post;
		$poll_postid = $post->ID;

		$poll_output = CBXPollHelper::show_single_poll_result($poll_postid, 'shortcode', 'text');

		echo $poll_output;
	}//end method metabox_result_display

	/**
	 * Renders metabox in right col to show  shortcode with copy to clipboard
	 */
	function metabox_shortcode_display()
	{
		global $post;
		$post_id = $post->ID;

		echo '<span data-clipboard-text=\'[cbxpoll id="' . intval($post_id) . '"]\' title="' . esc_html__("Click to clipboard", "cbxpoll") . '" id="cbxpollshortcode-' . intval($post_id) . '" class="cbxpollshortcode cbxpollshortcode-single cbxpollshortcode-' . intval($post_id) . '">[cbxpoll id="' . intval($post_id) . '"]</span>';
		echo '<div class="cbxpollclear"></div>';
	}//end method metabox_shortcode_display

	/**
	 * Save cbxpoll metabox
	 *
	 * @param $post_id
	 * @return bool
	 */
	function metabox_save($post_id)
	{
		// Check if our nonce is set.
		if (!isset($_POST['cbxpoll_meta_box_nonce'])) {
			return;
		}

		// Verify that the nonce is valid.
		if (!wp_verify_nonce($_POST['cbxpoll_meta_box_nonce'], 'cbxpoll_meta_box')) {
			return;
		}

		// If this is an autosave, our form has not been submitted, so we don't want to do anything.
		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
			return;
		}

		// Check the user's permissions.
		if (isset($_POST['post_type']) && 'cbxpoll' == $_POST['post_type']) {

			if (!current_user_can('edit_post', $post_id)) {
				return;
			}
		}

		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return $post_id;


		global $post;
		$post   = get_post($post_id);
		$status = $post->post_status;

		$prefix = '_cbxpoll_';

		//handle answer colors
		if (isset($_POST[$prefix . 'answer_color'])) {

			$colors = $_POST[$prefix . 'answer_color'];
			foreach ($colors as $index => $color) {
				$colors[$index] = CBXPollHelper::sanitize_hex_color($color);
			}

			$unique_color = array_unique($colors);

			if ((count($unique_color)) == (count($colors))) {
				update_post_meta($post_id, $prefix . 'answer_color', $colors);
			} else {
				$error = '<div class="error"><p>' . esc_html__('Error: Answer Color repeat error', 'cbxpoll') . '</p></div>';
				return false;
			}
		} else {
			delete_post_meta($post_id, $prefix . 'answer_color');
		}

		//handling extra fields
		if (isset($_POST[$prefix . 'answer_extra'])) {
			$extra = $_POST[$prefix . 'answer_extra'];
			update_post_meta($post_id, $prefix . 'answer_extra', $extra);

		} else {
			delete_post_meta($post_id, $prefix . 'answer_extra');
		}

		//handle answer titles
		if (isset($_POST[$prefix . 'answer'])) {
			$titles = $_POST[$prefix . 'answer'];

			foreach ($titles as $index => $title) {
				$titles[$index] = sanitize_text_field($title);
			}

			update_post_meta($post_id, $prefix . 'answer', $titles);
		} else {
			delete_post_meta($post_id, $prefix . 'answer');
		}

		$this->metabox_extra_save($post_id);
	}//end method metabox_save

	/**
	 * Save cbxpoll meta fields except poll color and titles
	 *
	 * @param $post_id
	 * @return bool|void
	 */
	function metabox_extra_save($post_id){
		//global $post_meta_fields;
		$post_meta_fields = CBXPollHelper::get_meta_fields();

		$prefix = '_cbxpoll_';


		$cb_date_array = array();
		foreach ($post_meta_fields as $field) {

			$old = get_post_meta($post_id, $field['id'], true);
			$new = $_POST[$field['id']];

			if (($prefix . 'start_date' == $field['id'] && $new == '') || ($prefix . 'end_date' == $field['id'] && $new == '')) {

				$cbpollerror = '<div class="notice notice-error inline"><p>' . esc_html__('Error:: Start or End date any one empty', 'cbxpoll') . '</p></div>';


				return false; //might stop processing here
			} else {


				update_post_meta($post_id, $field['id'], $new);

			}
		}
	}//end method metabox_extra_save

	/**
	 * Full reset CBX Poll
	 *
	 * This will not delete cbxpoll custom post types
	 */
	public function plugin_fullreset(){
		global $wpdb;

		$option_prefix = 'cbxpoll_';

		$option_values = CBXPollHelper::getAllOptionNames();

		foreach ($option_values as $key => $accounting_option_value ){
			delete_option($accounting_option_value['option_name']);
		}

		do_action('cbxpoll_plugin_option_delete');


		//delete tables

		$table_names = CBXPollHelper::getAllDBTablesList();
		$sql = "DROP TABLE IF EXISTS " . implode(', ', array_values($table_names));
		$query_result = $wpdb->query($sql);

		do_action('cbxpoll_plugin_table_delete');

		// create plugin's core table tables
		cbxpollHelper::install_table();

		//please note that, the default otpions will be created by default

		//reset total vote count meta for all cbxpoll type post
		$args = array(
			'post_type' => 'cbxpoll', // Only get the posts
			'posts_per_page'   => -1 // Get every post
		);
		$posts = get_posts($args);
		foreach ( $posts as $post ) {
			// Run a loop and update every meta data
			update_post_meta( $post->ID, '_cbxpoll_total_votes', 0 );
			update_post_meta($post->ID, '_cbxpoll_is_voted', 0);
		}
		//end reset total vote count meta for all cbxpoll type post

		//3rd party plugin's table creation
		do_action('cbxpoll_plugin_reset', $table_names, $option_prefix);



		$this->settings_api->set_sections($this->get_setting_sections());
		$this->settings_api->set_fields($this->get_setting_fields());
		$this->settings_api->admin_init();

		wp_safe_redirect(admin_url('edit.php?post_type=cbxpoll&page=cbxpollsetting#cbxpoll_tools'));
		exit();
	}

	/**
	 * Get Text answer templte
	 */
	public function get_answer_template(){

		//security check
		check_ajax_referer('cbxpoll', 'security');

		//get the fields

		$answer_counter   = intval($_POST['answer_counter']);
		$answer_color   = esc_attr($_POST['answer_color']);
		$is_voted       = intval($_POST['is_voted']);
		$poll_postid    = intval($_POST['poll_postid']);
		$answer_type    = esc_attr($_POST['answer_type']);

		$answers_extra = array('type' => $answer_type);

		$poll_answer = sprintf(esc_html__('Answer %d', 'cbxpoll'), ($answer_counter+1));

		$template = CBXPollHelper::cbxpoll_answer_field_template($answer_counter, $poll_answer, $answer_color, $is_voted, $answers_extra, $poll_postid);

		echo json_encode($template);
		die();
	}//end method get_answer_template

	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */
	public function plugin_listing_setting_link($links)
	{

		return array_merge(
			array(
				'settings' => '<a target="_blank" href="' . admin_url('edit.php?post_type=cbxpoll&page=cbxpollsetting') . '">' . esc_html__('Settings', 'cbxpoll') . '</a>'
			),
			$links
		);

	}//end method plugin_listing_setting_link

}//end class CBXPoll
