<?php
/**
 * Single poll widget functionality of the plugin.
 *
 * @link       https://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXPoll
 * @subpackage CBXPoll/widgets
 */

	// If this file is called directly, abort.
	if ( ! defined( 'WPINC' ) ) {
		die;
	}


	/**
	 * Single poll widget functionality of the plugin.
	 *
	 *
	 * @package    CBXPoll
	 * @subpackage CBXPoll/widgets
	 * @author     codeboxr <info@codeboxr.com>
	 */
	class CBXPollsingleWidget extends WP_Widget
	{

		/**
		 *
		 * Unique identifier for your widget.
		 *
		 *
		 * The variable name is used as the text domain when internationalizing strings
		 * of text. Its value should match the Text Domain file header in the main
		 * widget file.
		 *
		 * @since    1.0.0
		 *
		 * @var      string
		 */
		protected $widget_slug = 'cbxpollsingle'; //main parent plugin's language file


		public function __construct()
		{

			parent::__construct(
				$this->get_widget_slug(),
				esc_html__('CBXPoll Single Widget', 'cbxpoll'),
				array(
					'classname'   => $this->get_widget_slug() . '-class',
					'description' => esc_html__('Displays single poll', 'cbxpoll')
				)
			);

		} // end constructor


		/**
		 * Return the widget slug.
		 *
		 * @since    1.0.0
		 *
		 * @return    Plugin slug variable.
		 */
		public function get_widget_slug()
		{
			return $this->widget_slug;
		}

		/*--------------------------------------------------*/
		/* Widget API Functions
		/*--------------------------------------------------*/

		/**
		 * Outputs the content of the widget.
		 *
		 * @param array args  The array of form elements
		 * @param array instance The current instance of the widget
		 */
		public function widget($args, $instance)
		{
			$setting_api = new CBXPoll_Settings();
			//$global_answer_grid_list = $setting_api->get_option('answer_grid_list', 'cbxpoll_global_settings', 0); //0 = list 1 = grid

			extract($args, EXTR_SKIP);

			$widget_string = $before_widget;

			$title = apply_filters('widget_title', empty($instance['title']) ? esc_html__('CBXPoll Single Widget', 'cbxpoll') : $instance['title'], $instance, $this->id_base);
			// Defining the Widget Title
			if ($title) {
				$widget_string .= $args['before_title'] . $title . $args['after_title'];
			} else {
				$widget_string .= $args['before_title'] . $args['after_title'];
			}

			$instance = apply_filters('cbxpollsinglewidget_widget', $instance);

			$instance['chart_type']  = isset($instance['chart_type']) ? esc_attr($instance['chart_type']) : 'text';
			$instance['poll_id']     = isset($instance['poll_id']) ? intval($instance['poll_id']) : '';
			$instance['description'] = isset($instance['description']) ? intval($instance['description']) : 1;

			extract($instance);

			if (intval($poll_id) > 0 && (FALSE !== get_post_status( $poll_id ))) {
				$widget_string .= CBXPollHelper::cbxpoll_single_display($poll_id, 'widget', $chart_type, '', $description);
			} else {
				$widget_string .= '<p class="cbxpoll-voted-info cbxpoll_missing">' . esc_html__('Poll id missing or poll doesn\'t exists', 'cbxpoll') . '</p>';
			}

			$widget_string .= $after_widget;
			echo $widget_string;

		} // end widget




		/**
		 * Processes the widget's options to be saved.
		 *
		 * @param array new_instance The new instance of values to be generated via the update.
		 * @param array old_instance The previous instance of values before the update.
		 */
		public function update($new_instance, $old_instance)
		{

			$instance = $old_instance;

			$instance['title']       = sanitize_text_field($new_instance['title']);
			$instance['chart_type']  = esc_attr($new_instance['chart_type']);
			$instance['poll_id']     = intval($new_instance['poll_id']);
			$instance['description'] = intval($new_instance['description']);

			$instance = apply_filters('cbxpollsinglewidget_update', $instance, $new_instance);


			return $instance;

		} // end widget

		/**
		 * Generates the administration form for the widget.
		 *
		 * @param array instance The array of keys and values for the widget.
		 */
		public function form($instance)
		{

			$fields = array(
				'title'       => esc_html__('CBXPoll Single Widget', 'cbxpoll'),
				'chart_type'  => 'text', //chart type
				'poll_id'     => 0, //poll id,
				'description' => 1, //show description
			);

			$fields = apply_filters('cbxpollsinglewidget_widget_form_fields', $fields);

			$instance = wp_parse_args(
				(array)$instance,
				$fields
			);

			$instance = apply_filters('cbxpollsinglewidget_widget_form', $instance);

			extract($instance, EXTR_SKIP);


			// Display the admin form
			include(plugin_dir_path(__FILE__) . 'views/admin.php');

		} // end form
	} // end class